package com.ruoyi.system.service.impl;

import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.system.domain.ActSupplies;
import com.ruoyi.system.domain.ActSuppliesDetails;
import com.ruoyi.system.domain.ActSuppliesMaterialCode;
import com.ruoyi.system.mapper.AsiccSuppliesMapper;
import com.ruoyi.system.model.wages.dao.AsiccDAO;
import com.ruoyi.system.model.wages.vo.AsiccVo;
import com.ruoyi.system.service.AsiccSuppliesService;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.Range;
import org.springframework.stereotype.Service;

import javax.accessibility.AccessibleValue;
import java.lang.reflect.Array;
import java.util.*;

/**
 * @author haiwe
 * @date 2024/7/5
 */
@Service
@RequiredArgsConstructor
public class AsiccSuppliesServiceImpl implements AsiccSuppliesService {

    private final AsiccSuppliesMapper asiccSuppliesMapper;

    @Override
    public Map<String, Object> getData(AsiccDAO asiccDAO) {
        //处理特殊矿，如果那个表中有这个矿
        String mineId = asiccDAO.getMineId();
        List<ActSuppliesMaterialCode> materialCodeList = asiccSuppliesMapper.selectByMineId(mineId);

        // 使用Lambda表达式按codeMin的长度从长到短排序
        Comparator<ActSuppliesMaterialCode> comparator = Comparator.comparing(code -> code.getCodeMin().length());
        materialCodeList.sort(comparator.reversed());
        System.out.println(materialCodeList);
        Map<String, Object> voMap = new HashMap<>();
        //如果不存在这个矿信息，就走通用的规则
        List<ActSupplies> suppliesList;
        Map<Object, String> suppliesMap = new HashMap<>();
        if (materialCodeList.isEmpty()) {
            //查询一级分类，二级分类，以及对应的id
            suppliesList = asiccSuppliesMapper.selectLevelInfo();
            //存放suppliesList的一级分类和二级分类以及对应的id
            suppliesList.forEach(supplies -> {
                String oneLevel = supplies.getOneLevel();
                String twoLevel = supplies.getTwoLevel();
                Long id = supplies.getId();
                String finalLevel = oneLevel + "_" + twoLevel;
                suppliesMap.put(id, finalLevel);
            });
            List<ActSuppliesDetails> detailsList = asiccSuppliesMapper.select();
            //存放detailsList中的code
            Map<String, Object> codeMap = new HashMap<>();
            detailsList.forEach(v -> {
                String actCode = v.getCode();
                Long sid = v.getSid();
                codeMap.put(actCode, sid);
            });
            //特殊处理最后没有匹配上的数据的id为10
            ActSupplies actSupplies = asiccSuppliesMapper.selectByNum();
            //存放处理之后的code （asiccDAO中的Code）
            for (String code : asiccDAO.getCodes()) {
                Map<String, Object> map = new HashMap<>();
                //存放基本生产原煤的一级、二级分类
                Map<String, Object> codeList1 = new HashMap<>();
                //存放基本生产洗煤的一级、二级分类
                Map<String, Object> codeList2 = new HashMap<>();
                //存放其他费用来源的一级、二级分类
                Map<String, Object> codeList3 = new HashMap<>();
                //存放制造费用-原煤
                Map<String, Object> codeList4 = new HashMap<>();
                if (mineId.equals("010102")) {
                    String kmCode = StringUtils.substring(code, 0, 6);
                    if (codeMap.containsKey(kmCode)) {
                        get(voMap, suppliesMap, codeMap, code, map, codeList1, codeList2, codeList3,kmCode, codeList4);
                    } else {
                        kmCode = StringUtils.substring(kmCode, 0, 4);
                        if (codeMap.containsKey(kmCode)) {
                            getId(suppliesMap, codeMap, codeList1, codeList2, kmCode,codeList4);
                            codeList3.put("一级分类", "其他材料");
                            codeList3.put("二级分类", "其他");
                            map.put("基本生产原煤", codeList1);
                            map.put("基本生产洗煤", codeList2);
                            map.put("其他费用来源", codeList3);
                            map.put("制造费用原煤", codeList4);
                            voMap.put(code, map);
                        } else {
                            kmCode = StringUtils.substring(kmCode, 0, 2);
                            if (codeMap.containsKey(kmCode)) {
                                get(voMap, suppliesMap, codeMap, code, map, codeList1, codeList2, codeList3,kmCode, codeList4);
                            } else {
                                //kmCode一律为10
                                Common(actSupplies, map, codeList1, codeList2, codeList3);
                                codeList4.put("一级分类", "其他材料");
                                codeList4.put("二级分类", "其他");
                                map.put("制造费用原煤", codeList4);
                                voMap.put(code, map);
                            }
                        }
                    }
                } else {
                    Range<String> range = Range.between("30", "36");
                    String kmCode1 = StringUtils.substring(code, 0, 2);
                    String kmCode2 = StringUtils.substring(code, 0, 4);
                    if (mineId.equals("011704") && range.contains(kmCode1)) {
                        codeList1.put("一级分类", "配件");
                        codeList1.put("二级分类", "配件");
                        codeList2.put("一级分类", "配件");
                        codeList2.put("二级分类", "配件");
                        codeList3.put("一级分类", "其他材料");
                        codeList3.put("二级分类", "其他");
                        map.put("基本生产原煤", codeList1);
                        map.put("基本生产洗煤", codeList2);
                        map.put("其他费用来源", codeList3);
                        voMap.put(code, map);
                    } else if (mineId.equals("011704") && kmCode2.equals("6199")) {
                        codeList1.put("一级分类", "其他材料");
                        codeList1.put("二级分类", "其他");
                        codeList2.put("一级分类", "其他材料");
                        codeList2.put("二级分类", "其他");
                        codeList3.put("一级分类", "其他材料");
                        codeList3.put("二级分类", "其他");
                        map.put("基本生产原煤", codeList1);
                        map.put("基本生产洗煤", codeList2);
                        map.put("其他费用来源", codeList3);
                        voMap.put(code, map);
                    } else {
                        String kmCode = StringUtils.substring(code, 0, 6);
                        if (codeMap.containsKey(kmCode)) {
                            get(voMap, suppliesMap, codeMap, code, map, codeList1, codeList2, codeList3,kmCode, codeList4);
                        } else {
                            kmCode = StringUtils.substring(kmCode, 0, 4);
                            if (codeMap.containsKey(kmCode)) {
                                getId(suppliesMap, codeMap, codeList1, codeList2, kmCode,codeList4);
                                codeList3.put("一级分类", "其他材料");
                                codeList3.put("二级分类", "其他");
                                map.put("基本生产原煤", codeList1);
                                map.put("基本生产洗煤", codeList2);
                                map.put("其他费用来源", codeList3);
                                voMap.put(code, map);
                            } else {
                                kmCode = StringUtils.substring(kmCode, 0, 2);
                                if (codeMap.containsKey(kmCode)) {
                                    get(voMap, suppliesMap, codeMap, code, map, codeList1, codeList2, codeList3,kmCode, codeList4);
                                } else {
                                    //kmCode一律为10
                                    Common(actSupplies, map, codeList1, codeList2, codeList3);
                                    voMap.put(code, map);
                                }
                            }
                        }
                    }
                }
            }
        } else {
            suppliesList = asiccSuppliesMapper.select1();
            suppliesList.forEach(supplies -> {
                String oneLevel = supplies.getOneLevel();
                String oneName = supplies.getSuppliesName();
                suppliesMap.put(oneLevel, oneName);
            });
            for (String code : asiccDAO.getCodes()) {
                boolean matched = false;
                Map<String, Object> map = new HashMap<>();
                //存放基本生产原煤的一级、二级分类
                Map<String, Object> codeList1 = new HashMap<>();
                //存放基本生产洗煤的一级、二级分类
                Map<String, Object> codeList2 = new HashMap<>();
                //存放其他费用来源的一级、二级分类
                Map<String, Object> codeList3 = new HashMap<>();
                for (ActSuppliesMaterialCode materialCode : materialCodeList) {
                    String kmCode = code;
                    String codeMin = materialCode.getCodeMin();
                    String codeMax = materialCode.getCodeMax();
                    Range<String> range = Range.between(codeMin, codeMax);
                    if (range.contains(kmCode)) {
                        getLevelInfo(voMap, suppliesMap, code, map, codeList1, codeList2, codeList3, materialCode);
                        matched = true;
                        break; // 跳出内部循环
                    } else {
                        kmCode = StringUtils.substring(kmCode, 0, 6);
                        if (range.contains(kmCode)) {
                            getLevelInfo(voMap, suppliesMap, code, map, codeList1, codeList2, codeList3, materialCode);
                            matched = true;
                            break; // 跳出内部循环
                        } else {
                            kmCode = StringUtils.substring(kmCode, 0, 4);
                            String[] number = {"6101", "6102"};
                            if (range.contains(kmCode)) {
                                // 跳出内部循环
                                if (mineId.equals("011702") && Arrays.asList(number).contains(kmCode)) {
                                    String twoLevel = materialCode.getMaterialName();
                                    String codeType = materialCode.getMaterialCodeType();
                                    String oneLevel = suppliesMap.get(codeType);
                                    codeList1.put("一级分类", oneLevel);
                                    codeList1.put("二级分类", twoLevel);
                                    codeList2.put("一级分类", "油脂及乳化液");
                                    codeList2.put("二级分类", "油脂");
                                    codeList3.put("一级分类", "其他材料");
                                    codeList3.put("二级分类", "其他");
                                    map.put("基本生产原煤", codeList1);
                                    map.put("基本生产洗煤", codeList2);
                                    map.put("其他费用来源", codeList3);
                                } else {
                                    getLevelInfo(voMap, suppliesMap, code, map, codeList1, codeList2, codeList3, materialCode);
                                }
                                matched = true;
                                break; // 跳出内部循环
                            } else {
                                kmCode = StringUtils.substring(kmCode, 0, 2);
                                if (range.contains(kmCode)) {
                                    String[] numbers = {"30", "31", "32", "34", "35"};
                                    if (mineId.equals("011702") && Arrays.asList(numbers).contains(kmCode)) {
                                        String twoLevel = materialCode.getMaterialName();
                                        String codeType = materialCode.getMaterialCodeType();
                                        String oneLevel = suppliesMap.get(codeType);
                                        codeList1.put("一级分类", oneLevel);
                                        codeList1.put("二级分类", twoLevel);
                                        codeList2.put("一级分类", "配件");
                                        codeList2.put("二级分类", "配件");
                                        codeList3.put("一级分类", "其他材料");
                                        codeList3.put("二级分类", "其他");
                                        map.put("基本生产原煤", codeList1);
                                        map.put("基本生产洗煤", codeList2);
                                        map.put("其他费用来源", codeList3);
                                    } else {
                                        getLevelInfo(voMap, suppliesMap, code, map, codeList1, codeList2, codeList3, materialCode);
                                    }
                                    matched = true;
                                    break; // 跳出内部循环
                                }
                            }
                        }
                    }
                }
                if (!matched) {
                    codeList1.put("一级分类", "其他材料");
                    codeList1.put("二级分类", "其他");
                    codeList2.put("一级分类", "其他材料");
                    codeList2.put("二级分类", "其他");
                    codeList3.put("一级分类", "其他材料");
                    codeList3.put("二级分类", "其他");
                    map.put("基本生产原煤", codeList1);
                    map.put("基本生产洗煤", codeList2);
                    map.put("其他费用来源", codeList3);
                }
                voMap.put(code, map);
            }
        }
        return voMap;
    }

    /**
     * 处理特殊表不存在id，且最后匹配不出来，为10
     */
    private void Common(ActSupplies actSupplies, Map<String, Object> map, Map<String, Object> codeList1, Map<String, Object> codeList2, Map<String, Object> codeList3) {
        String oneLevel = actSupplies.getSuppliesName();
        codeList1.put("一级分类", oneLevel);
        codeList1.put("二级分类", null);
        if (oneLevel.equals("油脂") || oneLevel.equals("配件")) {
            codeList2.put("一级分类", oneLevel);
            codeList2.put("二级分类", null);
        } else {
            codeList2.put("一级分类", "其他材料");
            codeList2.put("二级分类", "其他");
        }
        codeList3.put("一级分类", "其他材料");
        codeList3.put("二级分类", "其他");
        map.put("基本生产原煤", codeList1);
        map.put("基本生产洗煤", codeList2);
        map.put("其他费用来源", codeList3);
    }

    /**
     * 返回各个费用来源对应的一级、二级分类（处理矿id存在的规则）
     */
    private void getLevelInfo(Map<String, Object> voMap, Map<Object, String> suppliesMap,
                              String code, Map<String, Object> map,
                              Map<String, Object> codeList1, Map<String, Object> codeList2,
                              Map<String, Object> codeList3, ActSuppliesMaterialCode materialCode) {
        String twoLevel = materialCode.getMaterialName();
        String codeType = materialCode.getMaterialCodeType();
        String oneLevel = suppliesMap.get(codeType);
        codeList1.put("一级分类", oneLevel);
        codeList1.put("二级分类", twoLevel);
        if (oneLevel.equals("油脂及乳化液") || oneLevel.equals("配件")) {
            codeList2.put("一级分类", oneLevel);
            codeList2.put("二级分类", twoLevel);
        } else {
            codeList2.put("一级分类", "其他材料");
            codeList2.put("二级分类", "其他");
        }
        codeList3.put("一级分类", "其他材料");
        codeList3.put("二级分类", "其他");
        map.put("基本生产原煤", codeList1);
        map.put("基本生产洗煤", codeList2);
        map.put("其他费用来源", codeList3);
        voMap.put(code, map);
    }

    /**
     * 返回各个费用来源对应的一级、二级分类（处理矿id不存在的规则）
     */
    private void get(Map<String, Object> voMap, Map<Object, String> suppliesMap,
                     Map<String, Object> codeMap, String code, Map<String, Object> map,
                     Map<String, Object> codeList1, Map<String, Object> codeList2,
                     Map<String, Object> codeList3, String kmCode,
                     Map<String,Object> codeList4) {
        getId(suppliesMap, codeMap, codeList1, codeList2, kmCode,codeList4);
        codeList3.put("一级分类", "其他材料");
        codeList3.put("二级分类", "其他");
        map.put("基本生产原煤", codeList1);
        map.put("基本生产洗煤", codeList2);
        map.put("其他费用来源", codeList3);
        map.put("制造费用原煤", codeList4);
        voMap.put(code, map);
    }

    /**
     * 处理原煤和洗煤的一级和二级分类（处理矿id不存在）
     */
    private void getId(Map<Object, String> suppliesMap, Map<String, Object> codeMap,
                       Map<String, Object> codeList1, Map<String, Object> codeList2, String kmCode,
                       Map<String,Object> codeList4) {
        Object sid = codeMap.get(kmCode);
        String finalLevel = suppliesMap.get(sid);
        String[] part = finalLevel.split("_");
        String oneLevel = part[0];
        String twoLevel = part[1];
        codeList1.put("一级分类", oneLevel);
        codeList1.put("二级分类", twoLevel);
        if (oneLevel.equals("油脂及乳化液") || oneLevel.equals("配件")) {
            codeList2.put("一级分类", oneLevel);
            codeList2.put("二级分类", twoLevel);
        } else {
            codeList2.put("一级分类", "其他材料");
            codeList2.put("二级分类", "其他");
        }

        if (oneLevel.equals("木材")||oneLevel.equals("支护用品")||oneLevel.equals("大型材料")||oneLevel.equals("劳保用品")) {
            codeList4.put("一级分类", oneLevel);
            codeList4.put("二级分类", twoLevel);
        } else {
            codeList4.put("一级分类", "其他材料");
            codeList4.put("二级分类", "其他");
        }
    }
}
